@@ -1,5 +1,5 @@
-# $Revision: 1.23 $
-# $Id: CPAN.pm,v 1.23 2003/03/17 05:22:14 afoxson Exp $
+# $Revision: 1.37 $
+# $Id: CPAN.pm,v 1.37 2003/03/23 05:03:36 afoxson Exp $
 
 # Bot::CPAN - provides CPAN services via IRC
 # Copyright (c) 2003 Adam J. Foxson. All rights reserved.
@@ -25,7 +25,7 @@ use Bot::CPAN::Glue;
 use vars qw(@ISA $VERSION);
 
 @ISA = qw(Bot::CPAN::Glue);
-$VERSION = '0.01_06';
+$VERSION = '0.01_07-pre';
 
 local $^W;
 
@@ -42,7 +42,11 @@ sub _add_to_recent {
 sub author :Private(notice) :Public(privmsg) :Args(required)
 :Help('retrieves the author of a module') {
 	my ($self, $event, $module) = @_;
-	$self->_print($event, $self->_get_details($module, 'Author'));
+	my $author = $self->_get_details($event, $module, 'Author');
+
+	return unless $author;
+
+	$self->_print($event, $author);
 }
 
 sub botsnack :Public(privmsg) :Args(refuse)
@@ -51,6 +55,84 @@ sub botsnack :Public(privmsg) :Args(refuse)
 	$self->_print($event, ':)');
 }
 
+sub _check_author {
+	my ($self, $author) = @_;
+	my $cp = $self->get('cp');
+
+	$author = uc $author;
+
+	return unless defined $cp->author_tree->{$author};
+	return $author;
+}
+
+sub _check_module {
+	my ($self, $event, $type, $module) = @_;
+	my $cp = $self->get('cp');
+	my $rv;
+
+	$rv = $self->_check_module_match($type, $module);
+
+	return ($rv, $module, 1) if $rv;
+
+	my $search = $cp->search(type => 'module', list => [qr/^$module$/i]);
+	my @cache;
+
+	for my $key (keys %{$search}) {
+		push @cache, $key;
+	}
+
+	if (scalar @cache == 1) {
+		$rv = $self->_check_module_match($type, $cache[0]);
+		return ($rv, $cache[0], 2) if $rv;
+	}
+
+	my @recent = @{$self->get('recent')};
+	for my $recent (@recent) {
+		$recent =~ s/-/::/g;
+		$recent =~ s/::\d.+//g;
+
+		if ($recent eq $module) {
+			$self->_print($event,
+				"$module is a brand new distribution. I'll have details shortly.");
+			return;
+		}
+	}
+
+	$type eq 'readme' ?
+		$self->_print($event, "No such module (or readme): $module") :
+		$self->_print($event, "No such module: $module");
+	return;
+}
+
+sub _check_module_match {
+	my ($self, $type, $module) = @_;
+	my $cp = $self->get('cp');
+	my $rv;
+
+	if ($type eq 'details') {
+		$rv = $cp->details(modules => [$module]);
+		return $rv if $rv->ok;
+	}
+	elsif ($type eq 'module_tree') {
+		$rv = $cp->module_tree->{$module};
+		return $rv if defined $rv;
+	}
+	elsif ($type eq 'pathname') {
+		$rv = $cp->pathname(to => $module);
+		return $rv if $rv;
+	}
+	elsif ($type eq 'readme') {
+		$rv = $cp->readme(modules => [$module]);
+		return $rv if $rv->ok;
+	}
+	elsif ($type eq 'reports') {
+		$rv = $cp->reports(modules => [$module]);
+		return $rv if defined $rv->rv;
+	}   
+
+	return;
+}
+
 sub _check_policy {
 	my ($self, $channel, $inform) = @_;
 	my $policy = $self->get('policy'); # by default is {}
@@ -74,24 +156,51 @@ sub _check_policy {
 	return 1 unless scalar keys %{$chanpol} > 0;
 
 	# we can assume that if an allow/deny exists it is defined
-	if (exists $chanpol->{allow} and $inform =~ /$chanpol->{allow}/i) {
+	if (exists $chanpol->{allow} and $inform =~ /$chanpol->{allow}/) {
 		return 1;
 	}
-	elsif (exists $chanpol->{deny} and $inform =~ /$chanpol->{deny}/i) {
+	elsif (exists $chanpol->{deny} and $inform =~ /$chanpol->{deny}/) {
 		return 0;
 	}
 
 	return 1;
 }
 
+sub config :Private(notice) :Args(refuse) :Admin :LowPrio
+:Help('Shows the bots configuration') {
+	my ($self, $event) = @_;
+
+	$self->_print($event, 'Adminhost: ' . $self->get('adminhost'));
+	$self->_print($event, 'Alt Nicks: ' . join ', ', $self->alt_nicks) if
+		scalar $self->alt_nicks > 0;
+	$self->_print($event, 'Channels: ' . join ', ', $self->channels);
+	$self->_print($event, 'Ignore List: ' . join ', ', $self->ignore_list) if
+		scalar $self->ignore_list > 0;
+	$self->_print($event, 'Debug: ' . $self->debug);
+	$self->_print($event, 'Group: ' . $self->get('group'));
+	$self->_print($event, 'Inform Channel of New Uploads Interval: ' .
+		$self->get('inform_channel_of_new_uploads'));
+	$self->_print($event, 'Name: ' . $self->name);
+	$self->_print($event, 'News Server: ' . $self->get('news_server'));
+	$self->_print($event, 'Nick: ' . $self->nick);
+	$self->_print($event, 'Port: ' . $self->port);
+	$self->_print($event, 'Reload Indice Interval: ' .
+		$self->get('reload_indices_interval'));
+	$self->_print($event, 'Search Max Results: ' .
+		$self->get('search_max_results'));
+	$self->_print($event, 'Servers: ' . join ', ', $self->servers);
+	$self->_print($event, 'Username: ' . $self->username);
+}
+
 # this is called the moment we successfully connect to a server
 sub connected {
 	my $self = shift;
-	my $cp = CPANPLUS::Backend->new();
+	my $cp = CPANPLUS::Backend->new() unless $self->get('cp');
 
-	$self->set('cp', $cp);
-	$self->set('requests', 0);
-	$self->set('recent', []);
+	$self->set('cp', $cp) unless $self->get('cp');
+	$self->set('requests', 0) unless $self->get('requests');
+	$self->set('recent', []) unless $self->get('recent') and
+		ref $self->get('recent') eq 'ARRAY';
 
 	my $nntp = Net::NNTP->new($self->get('news_server')) or
 		die "Cannot open NNTP server: $!";
@@ -110,61 +219,48 @@ sub connected {
 sub description :Private(notice) :Public(privmsg) :Args(required)
 :Help('retrieves the description of a module') {
 	my ($self, $event, $module) = @_;
-	$self->_print($event, $self->_get_details($module, 'Description'));
+	my $description = $self->_get_details($event, $module, 'Description');
+
+	return unless $description;
+
+	$self->_print($event, $description);
 }
 
 sub details :Private(notice) :Fork :LowPrio :Args(required)
 :Help('retrieves full details of a module') {
 	my ($self, $event, $module) = @_;
-	my $cp = $self->get('cp');
-	my $details = $cp->details(modules => [$module]);
+	my ($details, $actual) = $self->_check_module($event, 'details', $module);
+	return unless $details;
 
-	if (not defined $details->rv) {
-		$self->_print($event, "No such module: $module");
-	}
-	else {
-		for my $bit
-		(   
-			'Author', 'Description', 'Development Stage',
-			'Interface Style', 'Language Used', 'Package',
-			'Support Level', 'Version on CPAN',
-		)
-		{   
-			next if $details->rv->{$module}->{$bit} =~
-				/^Unknown|None given$/;
-			$self->_print($event, "$bit: " . $details->rv->{$module}->{$bit});
-		}
+	for my $bit (   
+		'Author', 'Description', 'Development Stage', 'Interface Style',
+		'Language Used', 'Package', 'Support Level', 'Version on CPAN',
+	)
+	{   
+		next if $details->rv->{$actual}->{$bit} =~ /^Unknown|None given$/;
+		$self->_print($event, "$bit: " . $details->rv->{$actual}->{$bit});
 	}
 }
 
 sub distributions :Private(notice) :Fork :LowPrio :Args(required)
 :Help('retrieves all of the distributions by an author') {
 	my ($self, $event, $author) = @_;
-
-	$author = uc $author;
-
-	if (length $author < 3) {
-		$self->_print($event, "Author ID '$author' is too small");
-		return;
-	}
-
 	my $cp = $self->get('cp');
-	my $auth_search = $cp->search(type => 'author', list => [$author],
-		authors_only => 1);
+	my $actual_author = $self->_check_author($author);
 
-	if (not defined $auth_search) {
+	if (not $actual_author) {
 		$self->_print($event, "No such author: $author");
 		return;
 	}
 
-	my $distributions = $cp->distributions(authors => [$author]);
+	my $distributions = $cp->distributions(authors => [$actual_author]);
 
 	if (not $distributions) {
-		$self->_print($event, "Author ID '$author' has no distributions");
+		$self->_print($event, "Author '$actual_author' has no distributions");
 		return;
 	}
 
-	for my $rpt (keys %{$distributions->rv->{$author}})
+	for my $rpt (keys %{$distributions->rv->{$actual_author}})
 	{
 		$rpt =~ s/\.tar\.gz$//;
 		$rpt =~ s/\.tgz$//;
@@ -173,15 +269,38 @@ sub distributions :Private(notice) :Fork :LowPrio :Args(required)
 	}
 }
 
-sub _get_details {
-	my ($self, $module, $type) = @_;
-	my $cp = $self->get('cp');
-	my $details = $cp->details(modules => [$module]);
+sub dlurl :Private(notice) :Public(privmsg) :Args(required)
+:Help('retrieves the download url of a module') {
+	my ($self, $event, $module) = @_;
+	my ($details) = $self->_check_module($event, 'module_tree', $module);
+	return unless $details;
 
-	return "No such module: $module" unless $details->ok;
-	return "No such module: $module" unless
-		$details->rv->{$module}->{$type};
-	return $details->rv->{$module}->{$type};
+	my $path = $details->path;
+	my $package = $details->package;
+	my $buffer = 'http://search.cpan.org/CPAN/authors/id/';
+
+	$buffer .= $path . '/' . $package;
+	$self->_print($event, $buffer);
+}
+
+sub _get_details {
+	my ($self, $event, $module, $type) = @_;
+	my ($details, $actual, $rv) =
+		$self->_check_module($event, 'details', $module);
+	return unless $details;
+
+	if ($rv == 1) {
+		return $details->rv->{$actual}->{$type};
+	}
+	elsif ($rv == 2) {
+		my $type_string = $type;
+		$type_string = lc $type_string;
+		$type_string =~ s/cpan/CPAN/;
+
+		(substr $actual, -1, 1) =~ /^s$/i ?
+			return "${actual}' $type_string is: ${\($details->rv->{$actual}->{$type})}" :
+			return "${actual}'s $type_string is: ${\($details->rv->{$actual}->{$type})}";
+	}
 }
 
 sub help :Private(notice) :LowPrio :Args(optional)
@@ -189,10 +308,10 @@ sub help :Private(notice) :LowPrio :Args(optional)
 	my ($self, $event, $command) = @_;
 	my (@public_and_private, @public, @private);
 
-	$self->_print($event, $self->nick() . ' is brought to you by ' .
-		__PACKAGE__ . " version $VERSION");
-
 	if (not $command) {
+		$self->_print($event, $self->nick() . ' is brought to you by ' .
+			__PACKAGE__ . " version $VERSION");
+
 		for my $command (sort $self->_commands()) {
 			if ($self->_private_command($command) &&
 				$self->_public_command($command)) {
@@ -221,37 +340,32 @@ sub help :Private(notice) :LowPrio :Args(optional)
 sub language :Private(notice) :Public(privmsg) :Args(required)
 :Help('retrieves the language of a module') {
 	my ($self, $event, $module) = @_;
-	$self->_print($event, $self->_get_details($module, 'Language Used'));
+	my $language = $self->_get_details($event, $module, 'Language Used');
+
+	return unless $language;
+
+	$self->_print($event, $language);
 }
 
 sub modules :Private(notice) :Fork :LowPrio :Args(required)
 :Help('retrieves the modules created by a given author') {
 	my ($self, $event, $author) = @_;
-
-	$author = uc $author;
-
-	if (length $author < 3) {
-		$self->_print($event, "Author ID '$author' is too small");
-		return;
-	}
-
 	my $cp = $self->get('cp');
-	my $auth_search = $cp->search(type => 'author', list => [$author],
-		authors_only => 1);
+	my $actual_author = $self->_check_author($author);
 
-	if (not defined $auth_search) {
+	unless ($actual_author) {
 		$self->_print($event, "No such author: $author");
 		return;
 	}
 
-	my $modules = $cp->modules(authors => [$author]);
+	my $modules = $cp->modules(authors => [$actual_author]);
 
 	if (not $modules->rv) {
-		$self->_print($event, "Author ID '$author' has no modules");
+		$self->_print($event, "Author ID '$actual_author' has no modules");
 		return;
 	}
 
-	for my $rpt (keys %{$modules->rv->{$author}})
+	for my $rpt (keys %{$modules->rv->{$actual_author}})
 	{   
 		$self->_print($event, "$rpt");
 	}
@@ -260,7 +374,11 @@ sub modules :Private(notice) :Fork :LowPrio :Args(required)
 sub package :Private(notice) :Public(privmsg) :Args(required)
 :Help('retrieves the package of a module') {
 	my ($self, $event, $module) = @_;
-	$self->_print($event, $self->_get_details($module, 'Package'));
+	my $package = $self->_get_details($event, $module, 'Package');
+
+	return unless $package;
+
+	$self->_print($event, $package);
 }
 
 sub _parse_article {
@@ -312,39 +430,22 @@ sub _parse_article {
 sub path :Private(notice) :Public(privmsg) :Args(required)
 :Help('retrieves the full CPAN path of a module') {
 	my ($self, $event, $module) = @_;
-	my $cp = $self->get('cp');
-	my $path = $cp->pathname(to => $module);
-
-	unless ($path) {
-		$self->_print($event, "No such module: $module");
-		return;
-	}
+	my ($path) = $self->_check_module($event, 'pathname', $module);
+	return unless $path;
 
 	$self->_print($event, "\$CPAN/authors/id$path");
 }
 
 sub readme :Public(privmsg) :Private(notice) :Args(required)
-:Help('sends README for module via DCC CHAT') {
+:Help('sends readme for module via DCC CHAT') {
 	my ($self, $event, $module) = @_;
-	my $cp = $self->get('cp');
-	my $mod = $cp->module_tree->{$module};
-
-	unless (defined $mod) {
-		$self->_print($event, "No such module: $module");
-		return;
-	}
-
-	my $readme = $cp->readme(modules => [$module]);
-
-	unless ($readme->ok) {
-		$self->_print($event, "No README for: $module");
-		return;
-	}
+	my ($readme, $actual) = $self->_check_module($event, 'readme', $module);
+	return unless $readme;
 
 	my $who = $event->{who};
 
-	$self->set("readme_$who", $module);
-	$self->_print($event, "Sending..");
+	$self->set("readme_$who", $actual);
+	$self->_print($event, "Sending readme for $actual..");
 	$self->dcc($who, 'CHAT');
 }
 
@@ -364,19 +465,14 @@ sub recent :Private(notice) :Public(privmsg) :Args(refuse)
 sub rt :Private(notice) :Public(privmsg) :Args(required)
 :Help('retrieves the RT www path to a module') {
 	my ($self, $event, $module) = @_;
-	my $cp = $self->get('cp');
-	my $url = $cp->module_tree->{$module};
+	my ($url, $actual) = $self->_check_module($event, 'module_tree', $module);
+	return unless $url;
+
 	my $buffer = 'http://rt.cpan.org/NoAuth/Bugs.html?Dist=';
-
-	unless (defined $url) {
-		$self->_print($event, "No such module: $module");
-		return;
-	}
-
 	my $package = $url->package();
 
 	unless ($package =~ s/\.tgz$|\.tar\.gz$|\.zip$//) {
-		$self->_print($event, "Unable to get url for: $module");
+		$self->_print($event, "Unable to get url for: $actual");
 		return;
 	}
 
@@ -411,7 +507,11 @@ sub search :Private(notice) :Args(required) :LowPrio
 sub stage :Private(notice) :Public(privmsg) :Args(required)
 :Help('retrieves the stage of a module') {
 	my ($self, $event, $module) = @_;
-	$self->_print($event, $self->_get_details($module, 'Development Stage'));
+	my $stage = $self->_get_details($event, $module, 'Development Stage');
+
+	return unless $stage;
+
+	$self->_print($event, $stage);
 }
 
 sub status :Private(notice) :Public(privmsg) :Args(refuse)
@@ -426,29 +526,38 @@ sub status :Private(notice) :Public(privmsg) :Args(refuse)
 sub style :Private(notice) :Public(privmsg) :Args(required)
 :Help('retrieves the style of a module') {
 	my ($self, $event, $module) = @_;
-	$self->_print($event, $self->_get_details($module, 'Interface Style'));
+	my $style = $self->_get_details($event, $module, 'Interface Style');
+
+	return unless $style;
+
+	$self->_print($event, $style);
 }
 
 sub support :Private(notice) :Public(privmsg) :Args(required)
 :Help('retrieves the support level of a module') {
 	my ($self, $event, $module) = @_;
-	$self->_print($event, $self->_get_details($module, 'Support Level'));
+	my $support = $self->_get_details($event, $module, 'Support Level');
+
+	return unless $support;
+
+	$self->_print($event, $support);
 }
 
 sub tests :Private(notice) :Fork :LowPrio :Args(required)
-:Help('retrieves the test results of a module') {
+:Help('retrieves the test reports of a module') {
 	my ($self, $event, $module) = @_;
-	my $cp = $self->get('cp');
-	my $report = $cp->reports(modules => [$module]);
+	my ($report, $actual) = $self->_check_module($event, 'reports', $module);
+	return unless $report;
 
-	if (not defined $report->rv) {
-		$self->_print($event, "No such module: $module");
-	}
-	elsif (not $report->rv->{$module}) {
-		$self->_print($event, "No test results for: $module");
+	if (not $report->rv->{$actual}) {
+		$self->_print($event, "No test reports for: $actual");
 	}
 	else {
-		for my $rpt (@{$report->rv->{$module}}) {
+		$self->_print($event, sprintf "%d test report%s for $actual",
+			scalar @{$report->rv->{$actual}},
+			@{$report->rv->{$actual}} == 1 ? '' : 's');
+
+		for my $rpt (@{$report->rv->{$actual}}) {
 			$self->_print($event, "$rpt->{grade} $rpt->{platform}");
 		}
 	}
@@ -457,20 +566,15 @@ sub tests :Private(notice) :Fork :LowPrio :Args(required)
 sub url :Private(notice) :Public(privmsg) :Args(required)
 :Help('retrieves the www path to a module') {
 	my ($self, $event, $module) = @_;
-	my $cp = $self->get('cp');
-	my $url = $cp->module_tree->{$module};
+	my ($url, $actual) = $self->_check_module($event, 'module_tree', $module);
+	return unless $url;
+
 	my $buffer = 'http://search.cpan.org/author/';
-
-	unless (defined $url) {
-		$self->_print($event, "No such module: $module");
-		return;
-	}
-
 	my $author  = $url->author();
 	my $package = $url->package();
 
 	unless ($package =~ s/\.tgz$|\.tar\.gz$|\.zip$//) {
-		$self->_print($event, "Unable to get url for: $module");
+		$self->_print($event, "Unable to get url for: $actual");
 		return;
 	}
 
@@ -481,20 +585,25 @@ sub url :Private(notice) :Public(privmsg) :Args(required)
 sub version :Private(notice) :Public(privmsg) :Args(required)
 :Help('retrieves the latest version of a module') {
 	my ($self, $event, $module) = @_;
-	$self->_print($event, $self->_get_details($module, 'Version on CPAN'));
+	my $version = $self->_get_details($event, $module, 'Version on CPAN');
+
+	return unless $version;
+
+	$self->_print($event, $version);
 }
 
 sub whois :Private(notice) :Public(privmsg) :Args(required)
 :Help('gets an author's name and email from a CPAN ID') {
 	my ($self, $event, $author) = @_;
 	my $cp = $self->get('cp');
-	my $cpanauthor = $cp->author_tree->{$author};
+	my $actual_author = $self->_check_author($author);
 
-	unless (defined $cpanauthor) {
-		$self->_print($event, "No such CPAN ID: $author");
+	unless ($actual_author) {
+		$self->_print($event, "No such author: $author");
 		return;
 	}
 
+	my $cpanauthor = $cp->author_tree->{$actual_author};
 	my $name = $cpanauthor->name;
 	my $email = $cpanauthor->email || 'no email';
 
@@ -558,15 +667,15 @@ sub irc_dcc_start
 	my $module = $self->get("readme_$who");
 
 	unless ($module) {
-		$self->log("$who requested a DCC CHAT, but I've no matching README\n");
+		$self->log("$who requested a DCC CHAT, but I've no matching readme\n");
 		return;
 	}
 
 	my $readme = $cp->readme(modules => [$module]);
 	my $length = length($readme->rv->{$module});
 
-	$self->log("Sending $who README for $module ($length)\n");
-	$self->set("readme_$who", '');
+	$self->log("Sending $who readme for $module ($length)\n");
+	$self->delete("readme_$who");
 	$self->dcc_chat($magic, $readme->rv->{$module});
 }
 
@@ -587,4 +696,28 @@ sub _reload_indices {
 		$self->get('reload_indices_interval'));
 }
 
+sub irc_disconnected_state { 
+	my ($this, $server) = @_[OBJECT, ARG0];
+	$this->log("Lost connection to server $server.\n");
+	$poe_kernel->delay('_reload_indices' => undef);
+	$poe_kernel->delay('_inform_channel_of_new_uploads' => undef);
+	$poe_kernel->delay('reconnect' => 60);
+}
+
+sub irc_error_state { 
+	my ($this, $err) = @_[OBJECT, ARG0];
+	$this->log("Server error occurred! $err\n");
+	$poe_kernel->delay('_reload_indices' => undef);
+	$poe_kernel->delay('_inform_channel_of_new_uploads' => undef);
+	$poe_kernel->delay('reconnect' => 60);
+}
+
+sub irc_socketerr_state {
+	my ($this, $err) = @_[OBJECT, ARG0];
+	$this->log("Socket error occurred: $err\n");
+	$poe_kernel->delay('_reload_indices' => undef);
+	$poe_kernel->delay('_inform_channel_of_new_uploads' => undef);
+	$poe_kernel->delay('reconnect' => 60);
+}
+
 1;
